"""
Auto-generate some posixAccount attribute values

Status:
Experimental => you have to understand what it internally does when enabling it!

$Id: posixautogen.py,v 1.3 2012/02/22 16:29:50 michael Exp $
"""

import ldap

from msbase import Unicode1stValueDict
from w2lapp.schema.plugins.nis import syntax_registry,UidNumber,IA5String


class HomeDirectory(IA5String):
  oid = 'HomeDirectory-oid'
  desc = 'Path of Unix home directory of the user'
  homeDirectoryTemplate = u'/home/%(uid)s'

  def formValue(self):
    form_value = IA5String.formValue(self)
    if not form_value:
      e = Unicode1stValueDict(self._entry)
      e.charset = self._ls.charset
      try:
        form_value = self.homeDirectoryTemplate % e
      except KeyError:
        pass
    return form_value

syntax_registry.registerAttrType(
  HomeDirectory.oid,[
    '1.3.6.1.1.1.1.3', # homeDirectory
  ]
)


class AutogenUIDNumber(UidNumber):
  oid = 'AutogenUIDNumber-oid'
  desc = 'numeric Unix-UID'
  inputSize = 12
  minNewValue = 10000L
  maxNewValue = 19999L

  def formValue(self):
    form_value = UidNumber.formValue(self)
    if not form_value and \
       'posixaccount' in set([oc.lower() for oc in self._entry['objectClass']]):
      ldap_result = self._ls.l.search_s(
        self._ls.getSearchRoot(self._dn),
        ldap.SCOPE_SUBTREE,
        '(&(objectClass=posixAccount)(uidNumber>=%s)(uidNumber<=%s))' % (
          AutogenUIDNumber.minNewValue,self.maxNewValue
        ),
        attrlist=['uid','uidNumber'],
      )
      uidnumber_set = set()
      for ldap_dn,ldap_entry in ldap_result:
        if ldap_dn==self._dn:
          return ldap_entry['uidNumber'][0].decode(self._ls.charset)
        else:
          uidnumber_set.add(int(ldap_entry['uidNumber'][0]))
      for uidnumber in xrange(AutogenUIDNumber.minNewValue,self.maxNewValue+1):
        if uidnumber in uidnumber_set:
          AutogenUIDNumber.minNewValue = uidnumber
        else:
          break
      return unicode(uidnumber)
    else:
      return form_value

syntax_registry.registerAttrType(
  AutogenUIDNumber.oid,[
    '1.3.6.1.1.1.1.0', # uidNumber
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))
